/*
 * 3pi Laser Pointer Follower
 * 
 *
 * http://www.pololu.com/docs/0J21
 * http://www.pololu.com
 * http://forum.pololu.com
 *
 */

// The 3pi include file must be at the beginning of any program that
// uses the Pololu AVR library and 3pi.
#include "inc/3pi.h"
#include "avr_context.h"
// This include file allows data to be stored in program space.  The
// ATmega168 has 16k of program space compared to 1k of RAM, so large
// pieces of static data should be stored in program space.
#include <avr/pgmspace.h>
#define ftofxa(x)       ((FXA)((x) * 1024.0F))
#define TIME_CONST 8192
#define CW 0
#define CCW 1

static FXA get_fulcrum(unsigned int *, U8 );
static FXA fxadiv( FXA, FXA);
static FXA fxamul( FXA, FXA);
static void rotate(FXA);

// This is the main function, where the code starts.  All C programs
// must have a main() function defined somewhere.
int main()
{

    unsigned int sensor[5];
    FXA error = 0;
    FXA integral = 0;
    FXA output = 0;
    pololu_3pi_init(TIME_CONST);
    lcd_init_printf();

	// This is the "main loop" - it will run forever.
	while(1)
	{
        clear();
        read_line_sensors(sensor, IR_EMITTERS_OFF);
        error = 2048 - get_fulcrum(sensor, 5);
        integral += fxamul(ftofxa(0.01F), error);
        output = fxamul(ftofxa(0.05F), error) + fxamul(ftofxa(0.1F), integral);
      

        
        if ((error > -500) && (error < 500))
        {
                set_motors(50,50);
        }
        else
        {
                rotate(output);  
        
        }

 
        delay_ms(10);

	}
  
    
}

static FXA get_fulcrum(unsigned int *sensor, U8 num)
{
    FXA fulcrum_num = 0;
    FXA fulcrum_denom = 0;
    FXA temp;
    unsigned char c;
    
    for ( c=0;c<num;c++)
    {
        temp = (TIME_CONST - *(sensor+c)) >> 3;
        if (temp < 50) temp = 1;
        fulcrum_num += temp*c;
        fulcrum_denom += temp;
    
    }
    
    
    temp = fxadiv( fulcrum_num, fulcrum_denom);
    
  
    return (temp);

}

static FXA fxadiv( FXA num, FXA denom)
{
    S32 temp;
    temp = ((S32)num * 1024) / denom;


    return ((FXA)temp);
}


static FXA fxamul( FXA t1, FXA t2)
{
    S32 temp;
    temp = ((S32)t1 * (S32)t2) / 1024;


    return ((FXA)temp);
}

static void rotate(FXA speed)
{
    if (speed < 0)
        set_motors(-speed, speed);
    else if (speed > 0)
        set_motors(-speed, speed);
    else
        set_motors(0,0);

}




